<?php 
/*
	Plugin Name: WooCommerce Bradesco
	Plugin URI: http://www.johnhenrique.com/woocommerce-bradesco
	Description: Plugin para integrar o Bradesco como meio de pagamento. Siga as <a href="http://www.johnhenrique.com/woocommerce-moip-transparente" title="instruções de uso" target="_blank">instruções de uso</a>.
	Version: 1.4
	Author: John-Henrique 
	Author URI: http://johnhenrique.com
	Requires at least: 3.3.2
	Tested up to: 3.5.1
	Text domain: WC_Bradesco
*/




add_action( 'plugins_loaded', 'init_woocommerce_bradesco' );

function init_woocommerce_bradesco() {
	
	
	if ( !class_exists( 'WC_Payment_Gateway' ) ) { return; }
	
	
    class WC_Gateway_Bradesco extends WC_Payment_Gateway {
    	
    	public function __construct(){
    		global $woocommerce;
    		
    		
			$this->plugin_url			= WP_PLUGIN_URL . "/" . plugin_basename( dirname(__FILE__));
			$this->versao				= '1.3';
			
	    	$this->id 					= 'bradesco';
	    	$this->method_title 		= 'Bradesco - TEF';
	    	$this->icon 				= apply_filters('woocommerce_bradesco_icon', $this->plugin_url . '/images/bradesco.png' );
	    	$this->has_fields 			= false;
	    	
	    	
			
			// Load the form fields.
			$this->init_form_fields();
			
			// Load the settings.
			$this->init_settings();
			
			// Define user set variables
			$this->title				= $this->settings['title'];
			$this->description 			= $this->settings['description'];
			$this->instrucoes 			= $this->settings['instrucoes'];
			$this->agencia				= $this->settings['agencia'];
			$this->conta				= $this->settings['conta'];
			$this->loja					= $this->settings['loja'];
			$this->assinatura			= $this->settings['assinatura'];
			$this->ambiente				= $this->settings['ambiente'];
			
            
			$this->debug				= $this->settings['debug'];	
			
			
			// URL para o ambiente do servidor
			if( $this->ambiente == 'teste' ){
				$this->ambiente = 'http://mupteste.comercioeletronico.com.br/sepsTransfer/'. $this->loja .'/prepara_pagto.asp?merchantid='. $this->loja .'&orderid={pedido_id}';
				//$this->ambiente = 'http://mupteste.comercioeletronico.com.br/sepsapplet/'. $this->loja .'/prepara_pagto.asp?merchantid='. $this->loja .'&orderid={pedido_id}';
			}else{
				$this->ambiente = 'http://mup.comercioeletronico.com.br/sepsapplet/'. $this->loja .'/prepara_pagto.asp?merchantid='. $this->loja .'&orderid={pedido_id}';
			}
			
			
			// Logs
			if ($this->debug=='yes') $this->log = new WC_Logger();
			

			// Valido?
			if ( !$this->verifica_moeda() || !$this->verifica_credenciais() )
				$this->enabled = false;
			
			

				
			
			if ( version_compare( WOOCOMMERCE_VERSION, '2.0', '<' ) ) {
				// Pre 2.0
				add_action( 'woocommerce_update_options_payment_gateways', array(&$this, 'process_admin_options'));
				add_action('init', array(&$this, 'notificacao' ));
				//add_action('init', array(&$this, 'fix_url_order_received' )); no work in WC 2.0
			} else {
				// 2.0
				add_action( 'woocommerce_update_options_payment_gateways_'. $this->id, array( $this, 'process_admin_options' ) );
				add_action( 'woocommerce_api_'. strtolower( get_class( $this ) ), array( $this, 'notificacao' ) );
			}

			// Actions
			add_action('woocommerce_receipt_'. $this->id,	array(&$this, 'receipt_page' ));
			add_action('admin_notices', array(&$this, 'check'));
			
			
			
			// Customer Emails
			add_action('woocommerce_email_before_order_table', array(&$this, 'email_instructions'), 10, 2);
    	}
    	
    	

		
		
		public function init_form_fields(){
			
			
			$this->form_fields = array(
				'enabled' => array(
					'title' => __( 'Habilitar/desabilitar', 'WC_MoIP_Transparente' ),
					'type' => 'checkbox',
					'label' => __( 'Habilitar Bradesco TEF', 'WC_MoIP_Transparente' ),
					'default' => 'yes'
				),
				'title' => array(
					'title' => __( 'Rótulo', 'WC_MoIP_Transparente' ),
					'type' => 'text',
					'description' => __( 'Isto controla o que o comprador vai ver na tela de pagamento.', 'WC_MoIP_Transparente' ),
					'default' => __( 'Bradesco - Transferência eletrônica', 'WC_MoIP_Transparente' )
				),
				'description' => array(
					'title' => __( 'Descrição', 'WC_MoIP_Transparente' ),
					'type' => 'textarea',
					'description' => __( 'Isto controla a descrição do serviço de cobrança na tela de opções de pagamento.', 'WC_MoIP_Transparente' ),
					'default' => __( 'Realize pagamento através de transferência eletrônica entre contas Bradesco.', 'WC_MoIP_Transparente' )
				),
				'instrucoes' => array(
					'title' => __( 'Instruções de pagamento', 'WC_MoIP_Transparente' ),
					'type' => 'textarea',
					'description' => __( 'Esta mensagem é enviada ao comprador após finalizar a compra.', 'WC_MoIP_Transparente' ),
					'default' => __( 'O pagamento deste pedido foi realizado através de Transferência eletrônica entre contas com segurança do Bradesco.', 'WC_MoIP_Transparente' )
				),
				'agencia' => array(
					'title' => __( 'Número da agência', 'WC_MoIP_Transparente' ),
					'type' => 'text',
					'description' => __( 'Informe o número da agência de sua loja no Bradesco.', 'WC_MoIP_Transparente' ),
					'default' => '0001'
				),
				'conta' => array(
					'title' => __( 'Número da conta', 'WC_MoIP_Transparente' ),
					'type' => 'text',
					'description' => __( 'Informe o número da conta de sua loja no Bradesco.', 'WC_MoIP_Transparente' ),
					'default' => '0000001'
				),
				'assinatura' => array(
					'title' => __( 'Assinatura digital', 'WC_MoIP_Transparente' ),
					'type' => 'text',
					'description' => sprintf(__( 'Informe o código da assinatura digital da sua loja no sistema Comércio Eletrônico Bradesco. Veja %sassinatura digital%s', 'WC_MoIP_Transparente' ), '<a href="http://johnhenrique.com/plugins/woocommerce-bradesco#assinatura" target="_blank">', '</a>'),
					'default' => '7B94A66D138FFD2AEBFBE3D8E0BDBD02700A5B793F017B0C8B40252C3680F9CD05C7EEB1A395C84916425DC318F7F07518D416194FB077AA47A8DBD2B35B2E4C376A23385FDD9B0AA3E7D0712B373B9EFD11028ADFC763B7EAD4A3E70FC8BE4A3CF4DA804E692A6173F4B81F568D7A9A38F663106149A34409B3C48147FBAA12'
				),
				'loja' => array(
					'title' => __( 'Código da loja', 'WC_MoIP_Transparente' ),
					'type' => 'text',
					'description' => sprintf(__( 'Informe o código de sua loja no sistema Comércio Eletrônico Bradesco. Veja %scódigo da loja%s', 'WC_MoIP_Transparente' ), '<a href="http://johnhenrique.com/plugins/woocommerce-bradesco#codigo-loja" target="_blank">', '</a>'),
					'default' => ''
				),
				'ambiente' => array(
					'title' => __( 'Ambiente', 'WC_MoIP_Transparente' ),
					'type' => 'select', 
					'options' => array(
						'teste' => __( "Testes", 'WC_MoIP_Transparente' ), 
						'' => __( "Produção", 'WC_MoIP_Transparente' ), 
					),
					'description' => sprintf(__( 'Informe em qual ambiente o Bradesco TEF deve trabalhar. Veja %sambiente%s.', 'WC_MoIP_Transparente' ), '<a href="http://johnhenrique.com/plugins/woocommerce-bradesco#ambiente" target="_blank">', '</a>'),
					'default' => 'teste'
				),
				'debug' => array(
					'title' => __( 'Depuração/Log', 'WC_MoIP_Transparente' ),
					'label'	=> __( 'Habilitar modo de depuração?' ), 
					'type' => 'checkbox',
					'description' => sprintf(__( 'Habilita o modo de depuração. Veja %sdepuração/log%s.', 'WC_MoIP_Transparente' ), '<a href="http://johnhenrique.com/plugins/woocommerce-bradesco#depuracao-log" target="_blank">', '</a>' ) .' (<code>woocommerce/logs/'. $this->id .'.txt</code>)',
					'default' => ''
				),
				
			);
			
		}
		
		
		
		public function admin_options(){
			?>
		    <h3><?php _e('Bradesco TEF', 'WC_MoIP_Transparente'); ?></h3>
			<p><?php _e('Preencha os campos abaixo para configurar o funcionamento do plugin e permitir que sua loja ofereça transferência entre contas Bradesco como meio de pagamento.', 'WC_MoIP_Transparente'); ?></p>
			<table class="form-table">
			<?php
				// Generate the HTML For the settings form.
				$this->generate_settings_html();
			?>
			</table>
			<?php
		} // End admin_options()
		
		
		
		/**
		 * Janela de pagamento
		 *
		 */
		public function payment_fields() {
			if ($this->description) echo wpautop(wptexturize($this->description));
		}
		
		
		
		/**
		 * Informa o final do processo de pagamento
		 * Exibe informações extras na página de 
		 * agradecimentos
		 *
		 * @param unknown_type $order_id
		 */
		public function thankyou_page() {
			if ($this->description) echo wpautop(wptexturize($this->description));
		}
		
			
		
		
		public function process_payment( $order_id ) {
			global $woocommerce;
			
			$order = &new WC_Order( $order_id );
			
			
			if($this->debug=='yes') $this->log->add(  $this->id, 'WooCommerce '. WOOCOMMERCE_VERSION );
			if($this->debug=='yes') $this->log->add(  $this->id, 'WooCommerce '. get_class( $this ) .' '. $this->versao );
			
			
			/**
			 * Em caso de erro
			 */
			$erro = false;
			if( $erro == true ){
				$woocommerce->add_error(__('Payment error:', 'WC_MoIP_Transparente') . $error_message); return;
			}
			
			
			// clean cart
			$woocommerce->cart->empty_cart();
			
			// no uncomment
			//$order->update_status('on-hold', __('Waiting payment', 'WC_MoIP_Transparente' ));
			
			
			// Empty awaiting payment session
			if ( version_compare( WOOCOMMERCE_VERSION, '2.0', '<' ) ) {
				// Pre 2.0
				unset($_SESSION['order_awaiting_payment']);
			} else {
			// 2.0
				unset( $woocommerce->session->order_awaiting_payment );
			}
			
			
			
			// Return thankyou redirect
			return array(
				'result'	=> 'success',
				'redirect'	=> add_query_arg('key', $order->order_key, add_query_arg('order', $order_id, get_permalink(get_option('woocommerce_pay_page_id'))))
			);
		}
		
		
		
		
		public function receipt_page(){
			global $woocommerce;
			
			$woocommerce->add_inline_js('
			jQuery(function(){
				jQuery("body").block(
					{ 
						message: "<img src=\"'.esc_url( $woocommerce->plugin_url() ).'/assets/images/ajax-loader.gif\" alt=\"'. __("Redirecting...", 'WC_PagSeguro') .'\" style=\"float:left; margin-right: 10px;\" />'.__('Thank you for your order. We are now redirecting you to Bradesco to make payment.', 'WC_PagSeguro').'", 
						overlayCSS: 
						{ 
							background: "#fff", 
							opacity: 0.6 
						},
						css: { 
					        padding:        20, 
					        textAlign:      "center", 
					        color:          "#555", 
					        border:         "3px solid #aaa", 
					        backgroundColor:"#fff", 
					        cursor:         "wait",
					        lineHeight:		"32px"
					    } 
					});
				jQuery("#submit_bradesco_payment_form").click();
			});');
			
			
			echo '<p>'.__('Obrigado por realizar sua compra. Clique no botão abaixo para realizar seu pagamento.', 'WC_MoIP_Transparente').'</p>';
			
			
			
			$imagens = $this->plugin_url . '/images';
			
			if( isset( $_GET['order'] ) ){
				
				$order = new WC_Order( $_GET['order'] );
			
				$url_payment = str_replace( '{pedido_id}', $order->id .'&key='. $order->order_key, $this->ambiente );
				
				
				
				if($this->debug=='yes') $this->log->add(  $this->id, 'URL: '. $url_payment );
			?>
			<form action="<?php echo $url_payment; ?>" method="POST">
				<input type="submit" value="Pagar" id="submit_bradesco_payment_form" >
			</form>
			<!--<a class="button" id="bradesco_payment_link" href="<?php echo $url_payment; ?>">pagar</a>-->
			<?php	
			}else{
				echo "Houve um erro, tente novamente";
			}
		}
		
		
		/**
		 * Retorna uma mensagem personalizada 
		 * sobre o método de pagamento
		 *
		 */
		public function email_instructions(){
			echo '<p><em>'. $this->instrucoes .'</em></p>';
		}
		
		
		
		/**
		 * Formata e corrige valores para somar o valor cobrado 
		 * com a taxa de emissão
		 *
		 * @param Integer $valor_cobrado
		 * @return Integer
		 */
		protected function helper_valor( $valor_cobrado = 0 ){
			return number_format( str_replace( ',', '.', $valor_cobrado ) + str_replace( ',', '.', $this->taxa ), 2, ',', '' );
		}
		
		
		/**
		 * Formata a string com base na 
		 * mascara informada
		 * 
		 * String: "3438420000", Máscara: "(##)####-####", Resultado: "(34)3842-0000";
		 * String: "12032010", Máscara: "##/##/##", Resultado: "12/03/2010";
		 * String: "2236", Máscara: "##:##", Resultado: "22:36".
		 * 
		 * @example mascara_string( "(##)####-####", $string );
		 *
		 * @param unknown_type $mascara
		 * @param unknown_type $string
		 * @return unknown
		 */
		protected function helper_mascara_string($mascara,$string){
		   $string = str_replace(" ","",$string);
		   for($i=0;$i<strlen($string);$i++){
		      $mascara[strpos($mascara,"#")] = $string[$i];
		   }
		   return $mascara;
		}
		
		
		
		/**
		 * Verifica pre requisitos e exibe um 
		 * flash com informações adicionais 
		 * caso algum problema seja encontrado
		 *
		 */
		public function check() {

			if ( !$this->verifica_moeda() ) :
				echo "<div class=\"inline error\">
					<p><strong>". __( 'Gateway Disabled', 'WC_MoIP_Transparente' ) ."</strong>: ". 
					sprintf( __( 'Você precisa definir a moeda Real brasileiro para utilizar o Bradesco TEF. Configure acessando %sGeral -> Opções gerais%s.', 'WC_MoIP_Transparente' ), '<a href="?page=woocommerce_settings">', '</a>' ) ."</p>
				</div>";
			endif;

			if ( !$this->helper_verifica_assinatura() ) :
				echo "<div class=\"inline error\">
					<p><strong>". __( 'Gateway Disabled', 'WC_MoIP_Transparente' ) ."</strong>: ". __( 'A Assinatura digital informada parece incorreta, ela precisa ter 256 digitos. Números e letras.', 'WC_MoIP_Transparente' ) ."</p>
				</div>";
			endif;

			if ( !$this->helper_verifica_agencia() ) :
				echo "<div class=\"inline error\">
					<p><strong>". __( 'Gateway Disabled', 'WC_MoIP_Transparente' ) ."</strong>: ". __( 'O código da agência parece incorreto, ele precisa ter 4 digitos. Somente números.', 'WC_MoIP_Transparente' ) ."</p>
				</div>";
			endif;

			if ( !$this->helper_verifica_conta() ) :
				echo "<div class=\"inline error\">
					<p><strong>". __( 'Gateway Disabled', 'WC_MoIP_Transparente' ) ."</strong>: ". __( 'O código da conta parece incorreto, ele precisa ter 7 digitos. Somente números.', 'WC_MoIP_Transparente' ) ."</p>
				</div>";
			endif;

			if ( !$this->helper_verifica_loja() ) :
				echo "<div class=\"inline error\">
					<p><strong>". __( 'Gateway Disabled', 'WC_MoIP_Transparente' ) ."</strong>: ". __( 'O código da loja parece incorreto. Somente números.', 'WC_MoIP_Transparente' ) ."</p>
				</div>";
			endif;
		}
		

		
		/**
		 * Verifica se a moeda padrão na loja 
		 * é Real brasileiro
		 *
		 * @since 1.0
		 */
		protected function verifica_moeda() {
			if (get_option('woocommerce_currency') == "BRL" ) {
				return true;
			}


			return false;
		}
		

		/**
		 * Verifica se as credenciais estão definidas
		 *
		 * @since 1.0
		 */
		protected function verifica_credenciais() {
			if ( empty( $this->conta ) || empty( $this->loja ) || empty( $this->assinatura ) || empty( $this->agencia ) )
				return false;
			else
				return true;
		}
		
		
		
		/**
		 * Verifica se a conta possui 256 digitos
		 *
		 * @since 1.0
		 * @return Bolean
		 */
		protected function helper_verifica_assinatura(){
			if( ( strlen( $this->assinatura ) != 256 ) || ( !ctype_alnum( $this->assinatura ) ) ){
				return false;
			}else{
				return true;
			}
		}
		
		
		/**
		 * Verifica se a agencia possui 4 digitos
		 *
		 * @since 1.0
		 * @return Bolean
		 */
		protected function helper_verifica_agencia(){
			if( ( strlen( $this->agencia ) != 4 ) || ( !ctype_digit( $this->agencia ) ) ){
				return false;
			}else{
				return true;
			}
		}
		
		
		
		/**
		 * Verifica se a conta possui 7 digitos
		 *
		 * @since 1.0
		 * @return Bolean
		 */
		protected function helper_verifica_conta(){
			if( ( strlen( $this->conta ) != 7 ) || ( !ctype_digit( $this->conta ) ) ){
				return false;
			}else{
				return true;
			}
		}
		
		
		/**
		 * Verifica se a loja
		 *
		 * @since 1.0
		 * @return Bolean
		 */
		protected function helper_verifica_loja(){
			if( ( strlen( $this->loja ) < 5 ) || ( !ctype_digit( $this->loja ) ) ){
				return false;
			}else{
				return true;
			}
		}
		
		
		
		/**
		 * Recebe os dados de status do pagamento
		 *
		 */
		public function notificacao() {
			global $woocommerce;

			//if($this->debug=='yes') $this->log->add(  $this->id, 'Resposta MUP: '. print_r( $_REQUEST, true ) );
				
			$dados = $_POST;
			
			
			// O que é isto?
			// transId => getTransfer
			// numOrder => ID do pedido
			if ( isset( $dados['transId'] ) && ( $dados['transId'] == 'getTransfer' ) && isset( $dados['numOrder'] ) ) {
				
				
				if($this->debug=='yes') $this->log->add(  $this->id, 'MUP GetTransfer: '. print_r( $_POST, true ) );

				// short cut
				$order_code 	= $dados['numOrder'];
				$order_tipo		= $dados['getTransfer'];
				$order_ID		= $dados['numOrder'];
				$order_status	= $dados['status_pagamento'];
				$order_total	= $dados['valtotal'];
				$assinatura		= $dados['assinatura'];
				$tipopagto		= $dados['tipopagto'];


				echo $this->envelope_xml();
			}
			
			
			
			if( $dados['transId'] == 'getOrder' ){
				if($this->debug=='yes') $this->log->add(  $this->id, 'MUP GetOrder: '. print_r( $_POST, true ) );
			}
			
			
			/**
			 * If request if a putAuth 
			 * confirm if payment data 
			 * is correct
			 */
			if( isset( $dados['transId'] ) && ( $dados['transId'] == 'putAuth' ) ){
				
				if($this->debug=='yes') $this->log->add(  $this->id, 'MUP PutAuth: '. print_r( $_POST, true ) );
				
				$order_ID 		= $dados['numOrder'];
				$order_key	 	= $dados['key'];
				$order_cod	 	= $dados['cod'];
				
				// this order exists?
				$pedido = new WC_Order( (int) $order_ID );
				
				
				/**
				 * For security reasons we check if the application
				 * exists and that the total order value is the
				 * same as the amount charged in store
				 *
				 * if any errors occur
				 * kill this process
				 */
				if ( !is_int( $pedido->id ) && !$pedido->key_is_valid( $order_key ) && ( $pedido->order_key != $order_key ) && ( $cod != 0 ) ) { //  && ( $pedido->order_total != $order_total )
					$woocommerce->add_error(__('Oops, there was an error during the payment process, please try again.', 'WC_MoIP_Transparente'));
					
					if($this->debug=='yes') $this->log->add( $this->id, 'MUP ERRO: '. print_r( $order_total, true ) );
					echo "<ERRO>";
					//exit;
				}else {

					// what is this?
					//$arrStatus = $this->helper_status( $order_status, $pedido );
					
					if($this->debug=='yes') $this->log->add( $this->id, 'MUP OK: '. print_r( $arrStatus, true ) );
					
					echo "<PUT_AUTH_OK>";
					//exit;
				}
			}
		}
		

		
		/**
		 * Prepara o pacote XML
		 *
		 * @return String XML
		 */
		protected function envelope_xml(){
			global $woocommerce;
						

			if($this->debug=='yes') $this->log->add(  $this->id, 'Dados: '. print_r( $_REQUEST, true ) );
			
			
			// Resgatando o ID na URL
			$order = new WC_Order( $_POST['numOrder'] );

			
			
			/** opcional mas não pode ser vazio 
						<adicional>=(frete)
						<valorAdicional>=(valorAdicional)
			*/
			$xml = '<BEGIN_ORDER_DESCRIPTION><orderid>=('. $order->id .')<descritivo>=(Pagamento para loja '. get_option( 'blogname' ) .')<quantidade>=(1)<unidade>=(un)<valor>=('. str_replace( array( ',', '.' ), '', $order->order_total ) .')<END_ORDER_DESCRIPTION><BEGIN_TRANSFER_DESCRIPTION><NUMEROAGENCIA>=('. $this->agencia .')<NUMEROCONTA>=('. $this->conta .')<ASSINATURA>=('. $this->assinatura .')<END_TRANSFER_DESCRIPTION>';
			
			
			
			if($this->debug=='yes') $this->log->add(  $this->id, 'Envelope XML: '. $xml );
			
			
			echo $xml;
			
			exit();
		}
		
		

			
		
    	
    }
}


function add_woocommerce_bradesco_gateway( $methods ) {
    $methods[] = 'WC_Gateway_Bradesco'; 
    return $methods;
}
 
add_filter( 'woocommerce_payment_gateways', 'add_woocommerce_bradesco_gateway' );

?>